#include "common.h"

void* 	my_alloc(unsigned int size)
{
	void *ptr;
	ptr = malloc(size);
	if (!ptr) 
	{
		fatal("memory allocation failed");
	}
	return ptr;
}

void* 	my_realloc(void* pptr, unsigned int size)
{
	void *ptr;
	ptr = realloc(pptr, size);
	if (!ptr) 
	{
		fatal("memory allocation failed");
	}
	return ptr;
}

void fatal(const char* str)
{
	fprintf(stderr, "*** fatal error: %s\n", str);
	write_log(log_name, wl_error, "fatal error: %s", str);
	write_log(log_name, wl_error, "exited");
	exit(EXIT_FAILURE);
}

int base64enc(char* in, int len, char* out)
{
	int i;
	int p1 = 1, p2 = 1;
	char c1, c2, c3;
	char b1, b2, b3, b4;
	char *ptr = in;
	char *oout = out;
	static int fc = 1;
	static char basetable[64];
	
	if (fc) 
	{
		char s;
		fc = 0;
		for (i = 0, s = 'A'; s <= 'Z'; i++, s++) 
			basetable[i] = s;
		for (s = 'a'; s <= 'z'; i++, s++)
			basetable[i] = s;
		for (s = '0'; s <= '9'; i++, s++) 
			basetable[i] = s;
		basetable[i++] = '+';
		basetable[i]   = '/';
	}
	
	while (ptr - in < len) 
	{
		c1 = *ptr++;
		c2 = (ptr - in) < len ? *ptr++ : (p1 = 0);
		c3 = (ptr - in) < len ? *ptr++ : (p2 = 0);
		b1 = ((c1&0xFC)>>2);
		b2 = ((c1&3)<<4)    + ((c2&0xF0)>>4);
		b3 = ((c2&0x0F)<<2) + ((c3&0xC0)>>6);
		b4 = (c3&0x3F);
		*out++ = basetable[b1]; 
		*out++ = basetable[b2]; 
		*out++ = basetable[b3];
		*out++ = basetable[b4];
		if (!p2) *--out   = '=';
		if (!p1) *--out   = '=';		
	}
	return ((len + (len%3 ? 3-(len%3) : 0)) / 3 * 4);
}

int
write_log(char *file_name, int reason, const char *fmt_str, ...)
{
	va_list argptr;
	char log_buff[1024];
	time_t	ctime;
	struct timeval tv;
	struct tm *lt;
	char *rs;
	FILE *log;
	int rc;

	if (!(opts.flags & OPT_LOG)) return 0;
	
	if ((log = fopen(file_name, "a")) == NULL) 
	{
		fatal("cant open log file");
	}
	
	gettimeofday(&tv, NULL);
	lt = localtime(&tv.tv_sec);
	sprintf(log_buff, "%.2d:%.2d:%.2d.%.2d [%.5d]", 
			lt->tm_hour, 
			lt->tm_min, 
			lt->tm_sec, 
			tv.tv_usec/10000L,
			getpid());

	switch (reason) 
	{
		case wl_debug:
			rs = " [dbg ] ";
			break;
		case wl_info:
			rs = " [info] ";
			break;
		case wl_error:
			rs = " [err ] ";
			break;
		default:
			rs = " [unkn] ";
			break;
	}
	strcat(log_buff, rs);

	if (fputs(log_buff, log) == EOF) 
	{
		fatal("cant write to log file");
	}
	
	va_start(argptr, fmt_str);
	rc = vfprintf(log, fmt_str, argptr);
	va_end(argptr);
	if (rc == EOF) 
	{
		fatal("cant write to log file");
	}
	fputc('\n', log);
	
	fclose(log);
	
	return 0;
}

int milisleep(int ms)
{
	struct timespec ts;
	if (ms > 1000)
	{
		sleep(ms/1000);
		ms %= 1000;
	}
	ts.tv_sec  = 0L;
	ts.tv_nsec = ms*1000000L;
	return nanosleep(&ts, NULL);
}
