#include "common.h"
#include "http.h"
#include "words.h"
#include "transf.h"

void sighandler(int);

int  max_http_errors 		= 20;
const char def_user_agent[] = "(null)";
char log_name[]				= "wpc.log";
static int exit_by_alarm;

static int check_auth(char *addr, const char* un, const char *pw);
static int check_loop(char *addr);
static int try_un_pw(char *addr, char *un, char *pw);

extern dictionary* def_passes;
pid_t  main_pid;

int 
process(char* addr)
{
	int rc, i;
	int sys_errors = 0;
	int sum_words = 0, j;
	int dict_words_proc;
	pid_t pid[MAX_PROC];
	int pid_status;

	signal(SIGALRM, sighandler);
	signal(SIGINT, sighandler);
	main_pid = getpid();
	
	exit_by_alarm = 1;
	rc = check_auth(addr, NULL, NULL);
	exit_by_alarm = 0;
	if (rc == 200) 
	{
		printf("%s is not password protected\n", addr);
		return 0;
	}
	if (rc != 401) 
	{
		return 0;
	}
	write_log(log_name, wl_info, "gonna check %s", addr);
	
	if ((dcount = load_dictionaries(dicts)) < 0) 
	{
		fprintf(stderr, "error: cant load dictionaries\n");
		return -1;
	}
	for (i = 0; i <= dcount; i++) 
	{
		DEBUGPR(("dict: %s, words: %d", dicts[i]->path, dicts[i]->word_count));
		write_log(log_name, wl_info, "dict: %s, words: %d", dicts[i]->path, dicts[i]->word_count);
	}
	init_transf();
	
	
	printf("\n");
	for (i = 0; i < opts.max_proc; i++) 
	{
		int di;
		for (di = 0, j = 0; dicts[j] && j <= dcount; di++, j++) 
		{
			if (dicts[j]->word_count < (opts.max_proc << 1)) 
			{
				if (!i) 
				{
					word_dist[di].dict_index = j;
					word_dist[di].start = 1;
					word_dist[di].end   = dicts[j]->word_count;
				}
				else 
				{
					di--;
					continue;
				}
			}
			else 
			{
				dict_words_proc = dicts[j]->word_count / opts.max_proc;
				word_dist[di].dict_index = j;
				word_dist[di].start = i * dict_words_proc + 1;
				word_dist[di].end   = word_dist[di].start + dict_words_proc- 1;
				if (i == opts.max_proc - 1) 
				{
					word_dist[di].end += dicts[j]->word_count % dict_words_proc;
				}
			}
		}
		word_dist[di].dict_index = -1;
		rc = fork();
		if (rc < 0) 
		{
			fprintf(stderr, "fork failed, %s\n", strerror(errno));
			return -1;
		}
		if (rc == 0) 
		{
			exit(check_loop(addr));
		}
		pid[i] = rc;
		
	}

	while (i)
	{
		rc = waitpid(-1, &pid_status, WNOHANG);
		if (rc)
		{
			write_log(log_name, wl_debug, "child %d exited", rc);
			i--;
			continue;
		}
		else
		{
			sleep(1);
		}
	}
	
	return 0;
	
}


int 
check_loop(char *addr)
{
	char guess_username[MAX_USERNAME_LEN];
	char guess_password[MAX_PASSWORD_LEN];
	static char saved_username[MAX_PASSWORD_LEN];
	int i, rc;

#if defined(DEBUG)
	for (i = 0; word_dist[i].dict_index != -1; i++) 
	{
		printf("*** [%d] %d: %d->%d\n", getpid(), i+1, word_dist[i].start, word_dist[i].end);
	}
#endif
		DEBUGPR(("before in loop"));
	while (get_un_pw(guess_username, guess_password) == 0) 
	{
		DEBUGPR(("in loop"));
		rc = try_un_pw(addr, guess_username, guess_password);
		if (rc == 0) break;
		if (rc == -2) goto finish;
		if ((opts.flags & OPT_DEF_PASSES) && strcmp(guess_username, saved_username))
		{
			int start = 1;
			char *pass;
			while (pass = get_word(&start, def_passes))
			{
				rc = try_un_pw(addr, guess_username, pass);
				if (rc == 0) goto finish;
			}
			strcpy(saved_username, guess_username);
		}
	}
finish:
	write_log(log_name, wl_info, "traffic: %d/%d in/out", get_bytes_read(), get_bytes_wrote());
	return EXIT_SUCCESS;
}

int
try_un_pw(char *addr, char *un, char *pw)
{
	static unsigned http_errors;
	int rc;
	
	printf("trying %s/%s                         \r", un, pw);
	fflush(stdout);
	if (!(opts.flags & OPT_LOG_NOCOMB))
		write_log(log_name, wl_info, "trying %s/%s", un, pw);
	rc = check_auth(addr, un, pw);
	if (rc == 200)
	{
		printf("\r                                                       ");
		printf("\n*** valid pass: %s/%s\n\n",  un, pw);
		write_log(log_name, wl_info, "*** valid pass: %s/%s", un, pw);
		/* return 0; */
	}
	if (rc != 200 && rc != 401)  http_errors++;
	if (http_errors > max_http_errors)
	{
		fprintf(stderr, "\nerror: maximum error count reached\n");
		return -2;
	}
	if (rc == 503)
	{
		write_log(log_name, wl_info, "error 503 occured, sleeping for 10 seconds");
		sleep(10);
	}
	milisleep(opts.req_delay);
	return -1;
}

int 
check_auth(char *addr, const char* un, const char *pw)
{
	static int fc = 1;
	static http_url url;
	static http_request request;
	static http_response resp;
	static char *paddr = NULL;
	int rc;
	
	if (fc || paddr != addr)
	{
		if (http_parse_url(addr, &url) < 0) 
		{
			fatal("malformed url");
		}
		if (opts.proxy_server[0])
		{
			strcpy(url.proxy_server, opts.proxy_server);
			url.proxy_port = opts.proxy_port;
		}
		http_init_request(&request, (opts.flags & OPT_HTTP_GET) ? HTTP_METHOD_GET : HTTP_METHOD_HEAD);
		request.url	= &url;
		strcpy(request.user_agent, def_user_agent);
		
		DEBUGPR(("Server:    %s", url.server));
		DEBUGPR(("Port:      %d", url.port));
		DEBUGPR(("Location:  %s", url.location));
		
		paddr = addr;
		fc = 0;
	}
	
	if (un && pw) 
	{
		strcpy(url.username, un);
		strcpy(url.password, pw);
	}
	else 
	{
		url.username[0] = 0;
		url.password[0] = 0;
	}
	
	if ((rc = http_send(&request, &resp)) < 0) 
	{
		switch (rc) 
		{
			case HG_ERR_SYS:
				fprintf(stderr, "system error: %s\n", strerror(errno));
				break;
			case HG_ERR_UNRESOLVED:
				fprintf(stderr, "error: unknown host\n");
				break;
			default:
				fprintf(stderr, "unknown error occured\n");
				break;
		}
		fatal("execution aborted");
	}
	if (resp.status_code == 200) 
	{
		return resp.status_code;
	}
	if (resp.status_code != 401) 
	{
		fprintf(stderr, "Page is not protected, but some other error occured\n"
			   "(response code %d [%s])\n", resp.status_code, resp.reason_phrase);
	}

	return resp.status_code;
}


void sighandler(int sig)
{
	write_log(log_name, wl_debug, "signal %d received", sig);
	if (sig == SIGALRM)
	{
		/* I know this is silly, will make normal timeouts soon */
		if (exit_by_alarm)
		{
			fprintf(stderr, "\nerror: connection timed out\n");
			write_log(log_name, wl_debug, "connection timed out, exited");
			exit(EXIT_FAILURE);
		}
		signal(SIGALRM, sighandler);
		return;
	}
	if (sig == SIGINT)
	{
		if (getpid() == main_pid) 
		{
			fprintf(stderr, "\n*interrupted*\n");
			write_log(log_name, wl_info, "program interrupted");
		}
		exit(EXIT_FAILURE);
	}
}
